
# line 2 "hoc.y"
#include "hoc.h"
#define	code2(c1,c2)	code(c1); code(c2);
#define	code3(c1,c2,c3)	code(c1); code(c2); code(c3);

# line 6 "hoc.y"
typedef union  {				/* stack type */
	Symbol	*sym;			/* symbol table pointer */
	Inst	*inst;			/* machine instruction */
	int	narg;			/* number of arguments */
} YYSTYPE;
# define NUMBER 257
# define VAR 258
# define BLTIN 259
# define UNDEF 260
# define WHILE 261
# define IF 262
# define ELSE 263
# define PRINT 264
# define STRING 265
# define FUNCTION 266
# define PROCEDURE 267
# define RETURN 268
# define FUNC 269
# define PROC 270
# define READ 271
# define ARG 272
# define OR 273
# define AND 274
# define GT 275
# define GE 276
# define LT 277
# define LE 278
# define EQ 279
# define NE 280
# define UNARYMINUS 281
# define NOT 282
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
typedef int yytabelem;
# define YYERRCODE 256

# line 115 "hoc.y"

#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>

char	*progname;			/* for error messages */
int	lineno = 0;
jmp_buf	begin;
int	indef;				/* inside a definition */
char	*infile;			/* input file name */
FILE	*fin;				/* input file pointer */
char	**gargv;			/* global argument list */
int	gargc;				/* global argument count */
int	c;				/* global for use by warning */

main(argc,argv)				/* hoc6 */
int	argc;
char	**argv;
{
	int	i, fpecatch();

	progname = argv[0];
	if (argc == 1) {		/* fake an argument list */
		static char *stdinonly[] = { "-" }; /* the fake list */
		gargv = stdinonly;	/* point to argument list */
		gargc = 1;		/* only one file (stdin) */
	} else {
		gargv = argv+1;		/* file names only */
		gargc = argc - 1;	/* number of file names */
	}
	init();
	while (moreinput()) run();
	return 0;
}

/*
 *				**** MORE INPUT ****
 *
 * This routine sees if there is another input file in the list.
 * If so, then it is opened, and 1 is returned.
 * If not, then 0 is returned
 *
 */

moreinput()
{
	if (gargc-- <= 0) return 0;	/* no files left */
	if (fin && fin != stdin) fclose(fin); /* close last file */
	infile = *gargv++;		/* file name */
	lineno = 1;			/* reset line number */
	if (strcmp(infile,"-")==0) {	/* standard in */
		fin = stdin;
		infile = 0;
	} else if ((fin = fopen(infile,"r")) == NULL) {
		fprintf(stderr, "%s: can't open %s\n", progname, infile);
		return moreinput();
	}
	return 1;
}

/*
 *				**** RUN ****
 *
 * This routine continues to call the parser until an yyparse() returns 0
 *
 */

run()					/* execute until EOF */
{
	setjmp(begin);
	signal(SIGFPE,fpecatch);
	for (initcode(); yyparse(); initcode()) execute(progbase);
}

yylex()					/* hoc6 */
{
	while ((c=getc(fin)) == ' ' || c == '\t');
	if (c == EOF) return 0;

	if (c == '.' || isdigit(c)) { /* a number */
		double d;
		ungetc(c,fin);
		fscanf(fin,"%lf", &d);
		yylval.sym = install("", NUMBER, d);
		return NUMBER;
	}

	if (isalpha(c)) {
		Symbol *s;
		char sbuf[256], *p = sbuf;
		do {
			if (p>=sbuf + sizeof(sbuf)-1) {
				*p = '\0';
				execerror("name too long",sbuf);
			}
			*p++ = c;
		} while ((c=getc(fin)) != EOF && isalnum(c));
		ungetc(c,fin);
		*p = '\0';		/* terminate string */
		if ((s=lookup(sbuf)) == 0)
			s = install(sbuf,UNDEF,0.0);
		yylval.sym = s;
		return s->type == UNDEF ? VAR : s->type;
	}

	if (c == '$') {			/* argument */
		int n = 0;
		while (isdigit(c=getc(fin))) n = (n * 10) + (c - '0');
		ungetc(c,fin);
		if (n==0) execerror("strange $...", (char *) 0);
		yylval.narg = n;
		return ARG;
	}

	if (c == '"') {			/* quoted string */
		char sbuf[256], *p, *emalloc();
		for (p = sbuf; (c=getc(fin)) != '"'; p++) {
			if (c=='\n' || c == EOF) execerror("missing quote","");
			if (p>=sbuf+sizeof(sbuf) -1) {
				*p = '\0';
				execerror("string too long",sbuf);
			}
			*p = backslash(c); /* assign next char */
		}
		*p = 0;			/* terminate string */
		yylval.sym = (Symbol *)emalloc(strlen(sbuf)+1);
		strcpy(yylval.sym,sbuf);
		return STRING;
	}

	switch(c) {
	case '>':		return follow('=', GE, GT);
	case '<':		return follow('=', LE, LT);
	case '=':		return follow('=', EQ, '=');
	case '!':		return follow('=', NE, NOT);
	case '|':		return follow('|', OR, '|');
	case '&':		return follow('&', AND, '&');
	case '\n':		lineno++; return '\n';
	default:		return c;
	}
}

yyerror(s)				/* called for yacc syntax error */
char *s;
{
	warning(s,(char *) 0);
}

warning(s,t)	/* print warning message */
char *s, *t;
{
	fprintf(stderr,"%s: %s", progname, s);
	if (t) fprintf(stderr," %s", t);
	if (infile) fprintf(stderr, " in %s", infile);
	fprintf(stderr," near line %d\n", lineno);
	while (c != '\n' && c != EOF) {
		c = getc(fin);		/* flush rest of line */
	}
}

execerror(s,t)	/* recover from run time error */
char *s, *t;
{
	warning(s,t);			/* print error message */
	if (fin == stdin) fflush(stdin); /* flush if from terminal ... */
	else fseek(fin,0L,2);		/* ... or flush rest of file */
	longjmp(begin,0);		/* and return to 'safe' place */
}

fpecatch()	/* catch floating point exception */
{
	execerror("floating point exception", (char *) 0);
}

/*
 *				**** FOLLOW ****
 *
 * This routine looks ahead one character, then puts it back if
 *	it is not what was expected.
 * It returns the first argument if the character was found,
 *	otherwise it returns the second
 *
 */

follow(expect, ifyes, ifno)
int expect, ifyes, ifno;
{
	int c = getc(fin);

	if (c == expect) return ifyes;
	ungetc(c,fin);
	return ifno;
}

/*
 *				**** BACKSLASH ****
 *
 * This routine allows special characters to be quoted with back slashes
 *
 */

backslash(c)
int c;
{
	char	*strchr();
	static	char	transtab[] = "b\bf\fn\nr\rt\t";

	if (c != '\\') return c;	/* normal char, just return */
	c = getc(fin);			/* get 'escaped' character */
	/* if in translation table, return next character */
	if (islower(c) && strchr(transtab,c)) return strchr(transtab,c)[1];
	return c;			/* return character */
}

/*
 *				**** DEFNONLY ****
 *
 * This routine is called when encountering tokens that belong only within
 *	a definition.
 * It will produce a warning if not in one of them guys.
 *
 */

defnonly(s)
char *s;
{
	if (!indef) execerror(s,"used outside definition");
}
yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 64
# define YYLAST 485
yytabelem yyact[]={

     2,    22,    46,   109,    33,    31,    23,    32,   114,    34,
    47,    48,    95,   104,    33,    33,    31,    92,    32,    34,
    34,   100,    35,    86,    51,    50,     5,   111,   115,   112,
    22,   113,   113,   110,   108,    23,    30,    99,    98,    94,
    85,    64,    63,    59,    58,    54,    44,    22,    29,   105,
    33,    31,    23,    32,    28,    34,    35,    27,    97,    33,
    31,    60,    32,    82,    34,    62,    35,    35,    33,    31,
    45,    32,    53,    34,     4,    81,    33,    31,     3,    32,
    49,    34,     1,    16,    17,    87,    15,    90,    93,    61,
    55,     0,     0,     0,     0,     0,     0,     0,    22,     0,
     0,     0,    35,    23,   106,     0,     0,     0,     0,    33,
    31,    35,    32,    17,    34,     0,   107,     0,     0,     0,
    35,     0,     0,     0,     0,     0,     0,     0,    35,     0,
    17,    22,    91,   120,    22,     0,    23,   116,   117,    23,
     0,   119,     0,     0,    33,    31,     0,    32,     0,    34,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    35,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,    35,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    18,    10,
    21,     0,    25,    26,     0,    14,     0,    19,    13,    12,
     0,     0,    20,    11,     0,    43,    42,    36,    37,    38,
    39,    40,    41,    24,     0,     0,     7,    18,    10,    21,
     0,    25,    26,     0,    14,     0,    19,    13,    12,     8,
     9,    20,    11,     0,    18,    10,    21,     0,    25,    26,
     0,    14,    24,    19,    13,    12,     0,     0,    20,    11,
     0,    43,    42,    36,    37,    38,    39,    40,    41,    24,
    43,    42,    36,    37,    38,    39,    40,    41,     0,    43,
    42,    36,    37,    38,    39,    40,    41,    43,    42,    36,
    37,    38,    39,    40,    41,    18,    10,    21,     0,     0,
     0,     0,     0,   103,    19,     0,     0,     0,     0,    20,
    11,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    24,    42,    36,    37,    38,    39,    40,    41,    18,    10,
    21,    18,    10,    21,     0,     0,    57,    19,     0,     0,
    19,     0,    20,    11,     0,    20,    11,     0,     0,     0,
    88,     0,     6,    24,     0,     0,    24,    36,    37,    38,
    39,    40,    41,    52,     0,    56,     0,     0,     0,     0,
     0,     0,     0,    65,    66,    67,     0,     0,     0,     0,
     0,     0,    68,    69,    70,    71,    72,    73,    74,    75,
    76,    77,    78,    79,    80,     0,     0,     0,     0,     0,
     0,    83,    84,     0,     0,     0,     0,     0,     0,     0,
    89,     0,     0,     0,     0,    96,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,   101,   102,     0,     0,
     0,     0,     0,     0,     0,   101,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,   118 };
yytabelem yypact[]={

 -1000,   -10, -1000,    47,    44,    38,    26,    36,  -256,  -256,
   -36,   -37,    94, -1000,    91,     3,     3, -1000, -1000, -1000,
     2,     1,    94,    94,    94, -1000, -1000, -1000, -1000, -1000,
 -1000,    94,    94,    94,    94,    94,    94,    94,    94,    94,
    94,    94,    94,    94, -1000, -1000, -1000, -1000, -1000, -1000,
    94,    94,    34, -1000,     0,   -21,    34, -1000,   -39,    94,
   -39,     7,    -1,  -246,    94,    17,   -72,   -72,   -28,   -28,
   -72,   -72,   -72,   -27,   -27,   -27,   -27,   -27,   -27,   102,
    67,    -2,    -3,    34,    34,    94,    58, -1000,    34,     8,
 -1000, -1000, -1000, -1000,    94,    -7,   -38, -1000,    -8,   -14,
   -12,    34,    34, -1000, -1000, -1000,  -255,   -13, -1000, -1000,
   -39,   -39, -1000,    94,   -39, -1000, -1000, -1000,    34, -1000,
 -1000 };
yytabelem yypgo[]={

     0,    26,    72,   370,    90,    89,    44,    86,    83,    45,
    13,    70,    21,    82,    78,    75,    63 };
yytabelem yyr1[]={

     0,    13,    13,    13,    13,    13,    13,    13,     2,     2,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     6,
     7,     8,     9,    10,     5,     5,     5,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     4,     4,     4,     4,    15,    14,    16,    14,    11,    11,
    11,    12,    12,    12 };
yytabelem yyr2[]={

     0,     0,     4,     6,     7,     7,     7,     7,     7,     7,
     3,     3,     5,    11,     5,     9,     9,    15,     7,     7,
     3,     3,     1,     1,     1,     4,     4,     3,     3,     3,
     2,    11,     9,     9,     7,     7,     7,     7,     7,     7,
     5,     7,     7,     7,     7,     7,     7,     7,     7,     5,
     3,     3,     7,     7,     1,    13,     1,    13,     2,     2,
     2,     1,     3,     7 };
yytabelem yychk[]={

 -1000,   -13,    10,   -14,    -2,    -1,    -3,   256,   269,   270,
   258,   272,   268,   267,   264,    -7,    -8,   123,   257,   266,
   271,   259,    40,    45,   282,   261,   262,    10,    10,    10,
    10,    43,    45,    42,    47,    94,   275,   276,   277,   278,
   279,   280,   274,   273,    10,   -11,   258,   266,   267,   -11,
    61,    61,    -3,    -2,    -9,    -4,    -3,   265,    -6,    40,
    -6,    -5,    -9,    40,    40,    -3,    -3,    -3,    -3,    -3,
    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
    -3,   -15,   -16,    -3,    -3,    40,    44,    -1,    -3,    -3,
    -1,   125,    10,    -1,    40,   258,    -3,    41,    40,    40,
   -12,    -3,    -3,   265,   -10,    41,   -10,   -12,    41,    41,
    41,    41,    41,    44,   263,    41,    -1,    -1,    -3,    -1,
   -10 };
yytabelem yydef[]={

     1,    -2,     2,     0,    30,     0,     0,     0,     0,     0,
    28,    29,    11,    22,     0,     0,     0,    24,    27,    22,
     0,     0,     0,     0,     0,    20,    21,     3,     4,     5,
     6,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     7,    54,    58,    59,    60,    56,
     0,     0,    12,    30,     0,    14,    50,    51,     0,     0,
     0,     0,     0,     0,     0,     0,    40,    49,    35,    36,
    37,    38,    39,    41,    42,    43,    44,    45,    46,    47,
    48,     0,     0,     8,     9,    61,     0,    23,    10,     0,
    23,    18,    25,    26,    61,     0,     0,    34,     0,     0,
     0,    62,    52,    53,    15,    19,    16,     0,    32,    33,
     0,     0,    13,     0,     0,    31,    55,    57,    63,    23,
    17 };
typedef struct { char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"NUMBER",	257,
	"VAR",	258,
	"BLTIN",	259,
	"UNDEF",	260,
	"WHILE",	261,
	"IF",	262,
	"ELSE",	263,
	"PRINT",	264,
	"STRING",	265,
	"FUNCTION",	266,
	"PROCEDURE",	267,
	"RETURN",	268,
	"FUNC",	269,
	"PROC",	270,
	"READ",	271,
	"ARG",	272,
	"=",	61,
	"OR",	273,
	"AND",	274,
	"GT",	275,
	"GE",	276,
	"LT",	277,
	"LE",	278,
	"EQ",	279,
	"NE",	280,
	"+",	43,
	"-",	45,
	"*",	42,
	"/",	47,
	"UNARYMINUS",	281,
	"NOT",	282,
	"^",	94,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"list : /* empty */",
	"list : list '\n'",
	"list : list defn '\n'",
	"list : list asgn '\n'",
	"list : list stmt '\n'",
	"list : list expr '\n'",
	"list : list error '\n'",
	"asgn : VAR '=' expr",
	"asgn : ARG '=' expr",
	"stmt : expr",
	"stmt : RETURN",
	"stmt : RETURN expr",
	"stmt : PROCEDURE begin '(' arglist ')'",
	"stmt : PRINT prlist",
	"stmt : while cond stmt end",
	"stmt : if cond stmt end",
	"stmt : if cond stmt end ELSE stmt end",
	"stmt : '{' stmtlist '}'",
	"cond : '(' expr ')'",
	"while : WHILE",
	"if : IF",
	"begin : /* empty */",
	"end : /* empty */",
	"stmtlist : /* empty */",
	"stmtlist : stmtlist '\n'",
	"stmtlist : stmtlist stmt",
	"expr : NUMBER",
	"expr : VAR",
	"expr : ARG",
	"expr : asgn",
	"expr : FUNCTION begin '(' arglist ')'",
	"expr : READ '(' VAR ')'",
	"expr : BLTIN '(' expr ')'",
	"expr : '(' expr ')'",
	"expr : expr '+' expr",
	"expr : expr '-' expr",
	"expr : expr '*' expr",
	"expr : expr '/' expr",
	"expr : expr '^' expr",
	"expr : '-' expr",
	"expr : expr GT expr",
	"expr : expr GE expr",
	"expr : expr LT expr",
	"expr : expr LE expr",
	"expr : expr EQ expr",
	"expr : expr NE expr",
	"expr : expr AND expr",
	"expr : expr OR expr",
	"expr : NOT expr",
	"prlist : expr",
	"prlist : STRING",
	"prlist : prlist ',' expr",
	"prlist : prlist ',' STRING",
	"defn : FUNC procname",
	"defn : FUNC procname '(' ')' stmt",
	"defn : PROC procname",
	"defn : PROC procname '(' ')' stmt",
	"procname : VAR",
	"procname : FUNCTION",
	"procname : PROCEDURE",
	"arglist : /* empty */",
	"arglist : expr",
	"arglist : arglist ',' expr",
};
#endif /* YYDEBUG */
/*	@(#)yaccpar	1.8	*/

/*
** Skeleton parser driver for yacc output
** Modified Oct. 1981 by David Prosser Bell Labs Murray Hill N.J.
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-1000)

/*
** global variables used by the parser
*/
YYSTYPE yyv[ YYMAXDEPTH ];	/* value stack */
int yys[ YYMAXDEPTH ];		/* state stack */

YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */



/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
int
yyparse()
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ YYMAXDEPTH ] )	/* room on stack? */
		{
			yyerror( "yacc stack overflow" );
			YYABORT;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( "syntax error" );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 4:
# line 29 "hoc.y"
{ code2(pop, STOP); return 1; } break;
case 5:
# line 30 "hoc.y"
{ code(STOP); return 1; } break;
case 6:
# line 31 "hoc.y"
{ code2(print, STOP); return 1; } break;
case 7:
# line 32 "hoc.y"
{ yyerrok; } break;
case 8:
# line 34 "hoc.y"
{ yyval.inst=yypvt[-0].inst; code3(varpush,(Inst)yypvt[-2].sym,assign); } break;
case 9:
# line 35 "hoc.y"
{
		defnonly("$"); code2(argassign,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst; } break;
case 10:
# line 39 "hoc.y"
{ code(pop); } break;
case 11:
# line 40 "hoc.y"
{ defnonly("return"); code(procret); } break;
case 12:
# line 41 "hoc.y"
{ defnonly("return");yyval.inst=yypvt[-0].inst;code(funcret); } break;
case 13:
# line 42 "hoc.y"
{
				yyval.inst = yypvt[-3].inst;code3(call, (Inst)yypvt[-4].sym, (Inst) yypvt[-1].narg); } break;
case 14:
# line 44 "hoc.y"
{ yyval.inst = yypvt[-0].inst; } break;
case 15:
# line 45 "hoc.y"
{	/* while - do loop */
		(yypvt[-3].inst)[1] = (Inst)yypvt[-1].inst;	/* body of loop */
		(yypvt[-3].inst)[2] = (Inst)yypvt[-0].inst; } break;
case 16:
# line 48 "hoc.y"
{	/* if w/o else */
		(yypvt[-3].inst)[1] = (Inst)yypvt[-1].inst;	/* then part */
		(yypvt[-3].inst)[3] = (Inst)yypvt[-0].inst; } break;
case 17:
# line 51 "hoc.y"
{ /* if-then-else */
		(yypvt[-6].inst)[1] = (Inst)yypvt[-4].inst;	/* then part */
		(yypvt[-6].inst)[2] = (Inst)yypvt[-1].inst;	/* else part */
		(yypvt[-6].inst)[3] = (Inst)yypvt[-0].inst; } break;
case 18:
# line 55 "hoc.y"
{ yyval.inst = yypvt[-1].inst; } break;
case 19:
# line 57 "hoc.y"
{ code(STOP); yyval.inst = yypvt[-1].inst; } break;
case 20:
# line 59 "hoc.y"
{ yyval.inst = code3(whilecode, LATER, LATER); } break;
case 21:
# line 61 "hoc.y"
{ yyval.inst=code(ifcode);
				code3(LATER,LATER,LATER); } break;
case 22:
# line 64 "hoc.y"
{ yyval.inst = progp; } break;
case 23:
# line 66 "hoc.y"
{ code(STOP); yyval.inst = progp; } break;
case 24:
# line 68 "hoc.y"
{ yyval.inst = progp; } break;
case 27:
# line 72 "hoc.y"
{ yyval.inst = code2(constpush, (Inst)yypvt[-0].sym); } break;
case 28:
# line 73 "hoc.y"
{ yyval.inst = code3(varpush, (Inst)yypvt[-0].sym, eval); } break;
case 29:
# line 74 "hoc.y"
{ defnonly("$"); yyval.inst = code2(arg, (Inst)yypvt[-0].narg); } break;
case 31:
# line 76 "hoc.y"
{
				yyval.inst = yypvt[-3].inst; code3(call,(Inst)yypvt[-4].sym,(Inst)yypvt[-1].narg); } break;
case 32:
# line 78 "hoc.y"
{ yyval.inst = code2(varread, (Inst)yypvt[-1].sym); } break;
case 33:
# line 79 "hoc.y"
{ yyval.inst=yypvt[-1].inst; code2(bltin, (Inst)yypvt[-3].sym->u.ptr); } break;
case 34:
# line 80 "hoc.y"
{ yyval.inst = yypvt[-1].inst; } break;
case 35:
# line 81 "hoc.y"
{ code(add); } break;
case 36:
# line 82 "hoc.y"
{ code(sub); } break;
case 37:
# line 83 "hoc.y"
{ code(mul); } break;
case 38:
# line 84 "hoc.y"
{ code(div); } break;
case 39:
# line 85 "hoc.y"
{ code(power); } break;
case 40:
# line 86 "hoc.y"
{ yyval.inst = yypvt[-0].inst; code(negate); } break;
case 41:
# line 87 "hoc.y"
{ code(gt); } break;
case 42:
# line 88 "hoc.y"
{ code(ge); } break;
case 43:
# line 89 "hoc.y"
{ code(lt); } break;
case 44:
# line 90 "hoc.y"
{ code(le); } break;
case 45:
# line 91 "hoc.y"
{ code(eq); } break;
case 46:
# line 92 "hoc.y"
{ code(ne); } break;
case 47:
# line 93 "hoc.y"
{ code(and); } break;
case 48:
# line 94 "hoc.y"
{ code(or); } break;
case 49:
# line 95 "hoc.y"
{ yyval.inst = yypvt[-0].inst; code(not); } break;
case 50:
# line 97 "hoc.y"
{ code(prexpr); } break;
case 51:
# line 98 "hoc.y"
{ yyval.inst = code2(prstr, (Inst)yypvt[-0].sym); } break;
case 52:
# line 99 "hoc.y"
{ code(prexpr); } break;
case 53:
# line 100 "hoc.y"
{ code2(prstr, (Inst)yypvt[-0].sym); } break;
case 54:
# line 102 "hoc.y"
{ yypvt[-0].sym->type=FUNCTION; indef=1; } break;
case 55:
# line 103 "hoc.y"
{ code(procret); define(yypvt[-4].sym); indef=0; } break;
case 56:
# line 104 "hoc.y"
{ yypvt[-0].sym->type=PROCEDURE; indef=1; } break;
case 57:
# line 105 "hoc.y"
{ code(procret); define(yypvt[-4].sym); indef=0; } break;
case 61:
# line 111 "hoc.y"
{ yyval.narg = 0; } break;
case 62:
# line 112 "hoc.y"
{ yyval.narg = 1; } break;
case 63:
# line 113 "hoc.y"
{ yyval.narg = yypvt[-2].narg + 1; } break;
	}
	goto yystack;		/* reset registers in driver code */
}
