%{
#include "hoc.h"
#define	code2(c1,c2)	code(c1); code(c2);
#define	code3(c1,c2,c3)	code(c1); code(c2); code(c3);
%}
%union {				/* stack type */
	Symbol	*sym;			/* symbol table pointer */
	Inst	*inst;			/* machine instruction */
	int	narg;			/* number of arguments */
}
%token	<sym>	NUMBER VAR BLTIN UNDEF WHILE IF ELSE PRINT
%token	<sym>	STRING FUNCTION PROCEDURE RETURN FUNC PROC READ
%token	<narg>	ARG
%type	<inst>	stmt asgn expr prlist stmtlist cond while if begin end
%type	<sym>	procname
%type	<narg>	arglist
%right	'='
%left	OR
%left	AND
%left	GT GE LT LE EQ NE
%left	'+' '-'				/* left assoc, same precedence */
%left	'*' '/'				/* left assoc, higher preced. */
%left	UNARYMINUS NOT			/* unary minus */
%right	'^'				/* exponentiation */
%%
list:	/* nothing */
	| list '\n'
	| list defn '\n'
	| list asgn '\n'	{ code2(pop, STOP); return 1; }
	| list stmt '\n'	{ code(STOP); return 1; }
	| list expr '\n'	{ code2(print, STOP); return 1; }
	| list error '\n'	{ yyerrok; }
	;
asgn:	  VAR '=' expr		{ $$=$3; code3(varpush,(Inst)$1,assign); }
	| ARG '=' expr {
		defnonly("$"); code2(argassign,(Inst)$1); $$=$3; }
				
	;
stmt:	  expr			{ code(pop); }
	| RETURN		{ defnonly("return"); code(procret); }
	| RETURN expr		{ defnonly("return");$$=$2;code(funcret); }
	| PROCEDURE begin '(' arglist ')' {
				$$ = $2;code3(call, (Inst)$1, (Inst) $4); }
	| PRINT prlist		{ $$ = $2; }
	| while cond stmt end	{	/* while - do loop */
		($1)[1] = (Inst)$3;	/* body of loop */
		($1)[2] = (Inst)$4; }	/* end of loop */
	| if cond stmt end	{	/* if w/o else */
		($1)[1] = (Inst)$3;	/* then part */
		($1)[3] = (Inst)$4; }	/* end, if cond fails */
	| if cond stmt end ELSE stmt end { /* if-then-else */
		($1)[1] = (Inst)$3;	/* then part */
		($1)[2] = (Inst)$6;	/* else part */
		($1)[3] = (Inst)$7; }	/* end, if cond fails */
	| '{' stmtlist '}'	{ $$ = $2; }
	;
cond:	  '(' expr ')'		{ code(STOP); $$ = $2; }
	;
while:	  WHILE			{ $$ = code3(whilecode, LATER, LATER); }
	;
if:	  IF			{ $$=code(ifcode);
				code3(LATER,LATER,LATER); }
	;
begin:	  /* nothing */		{ $$ = progp; }
	;
end:	  /* nothing */		{ code(STOP); $$ = progp; }
	;
stmtlist: /* nothing */		{ $$ = progp; }
	| stmtlist '\n'
	| stmtlist stmt
	;
expr:	  NUMBER		{ $$ = code2(constpush, (Inst)$1); }
	| VAR			{ $$ = code3(varpush, (Inst)$1, eval); }
	| ARG			{ defnonly("$"); $$ = code2(arg, (Inst)$1); }
	| asgn
	| FUNCTION begin '(' arglist ')' {
				$$ = $2; code3(call,(Inst)$1,(Inst)$4); }
	| READ '(' VAR ')'	{ $$ = code2(varread, (Inst)$3); }
	| BLTIN '(' expr ')'	{ $$=$3; code2(bltin, (Inst)$1->u.ptr); }
	| '(' expr ')'		{ $$ = $2; }
	| expr '+' expr		{ code(add); }
	| expr '-' expr		{ code(sub); }
	| expr '*' expr		{ code(mul); }
	| expr '/' expr		{ code(div); }
	| expr '^' expr		{ code(power); }
	| '-' expr %prec UNARYMINUS	{ $$ = $2; code(negate); }
	| expr GT expr		{ code(gt); }
	| expr GE expr		{ code(ge); }
	| expr LT expr		{ code(lt); }
	| expr LE expr		{ code(le); }
	| expr EQ expr		{ code(eq); }
	| expr NE expr		{ code(ne); }
	| expr AND expr		{ code(and); }
	| expr OR expr		{ code(or); }
	| NOT expr		{ $$ = $2; code(not); }
	;
prlist:	  expr			{ code(prexpr); }
	| STRING		{ $$ = code2(prstr, (Inst)$1); }
	| prlist ',' expr	{ code(prexpr); }
	| prlist ',' STRING	{ code2(prstr, (Inst)$3); }
	;
defn:	  FUNC procname		{ $2->type=FUNCTION; indef=1; }
		'(' ')' stmt	{ code(procret); define($2); indef=0; }
	| PROC procname		{ $2->type=PROCEDURE; indef=1; }
		'(' ')' stmt	{ code(procret); define($2); indef=0; }
	;
procname:  VAR
	| FUNCTION
	| PROCEDURE
	;
arglist:	/* nothing */	{ $$ = 0; } /* number of args */
	| expr			{ $$ = 1; }
	| arglist ',' expr	{ $$ = $1 + 1; }
	;
%%
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>

char	*progname;			/* for error messages */
int	lineno = 0;
jmp_buf	begin;
int	indef;				/* inside a definition */
char	*infile;			/* input file name */
FILE	*fin;				/* input file pointer */
char	**gargv;			/* global argument list */
int	gargc;				/* global argument count */
int	c;				/* global for use by warning */

main(argc,argv)				/* hoc6 */
int	argc;
char	**argv;
{
	int	i, fpecatch();

	progname = argv[0];
	if (argc == 1) {		/* fake an argument list */
		static char *stdinonly[] = { "-" }; /* the fake list */
		gargv = stdinonly;	/* point to argument list */
		gargc = 1;		/* only one file (stdin) */
	} else {
		gargv = argv+1;		/* file names only */
		gargc = argc - 1;	/* number of file names */
	}
	init();
	while (moreinput()) run();
	return 0;
}

/*
 *				**** MORE INPUT ****
 *
 * This routine sees if there is another input file in the list.
 * If so, then it is opened, and 1 is returned.
 * If not, then 0 is returned
 *
 */

moreinput()
{
	if (gargc-- <= 0) return 0;	/* no files left */
	if (fin && fin != stdin) fclose(fin); /* close last file */
	infile = *gargv++;		/* file name */
	lineno = 1;			/* reset line number */
	if (strcmp(infile,"-")==0) {	/* standard in */
		fin = stdin;
		infile = 0;
	} else if ((fin = fopen(infile,"r")) == NULL) {
		fprintf(stderr, "%s: can't open %s\n", progname, infile);
		return moreinput();
	}
	return 1;
}

/*
 *				**** RUN ****
 *
 * This routine continues to call the parser until an yyparse() returns 0
 *
 */

run()					/* execute until EOF */
{
	setjmp(begin);
	signal(SIGFPE,fpecatch);
	for (initcode(); yyparse(); initcode()) execute(progbase);
}

yylex()					/* hoc6 */
{
	while ((c=getc(fin)) == ' ' || c == '\t');
	if (c == EOF) return 0;

	if (c == '.' || isdigit(c)) { /* a number */
		double d;
		ungetc(c,fin);
		fscanf(fin,"%lf", &d);
		yylval.sym = install("", NUMBER, d);
		return NUMBER;
	}

	if (isalpha(c)) {
		Symbol *s;
		char sbuf[256], *p = sbuf;
		do {
			if (p>=sbuf + sizeof(sbuf)-1) {
				*p = '\0';
				execerror("name too long",sbuf);
			}
			*p++ = c;
		} while ((c=getc(fin)) != EOF && isalnum(c));
		ungetc(c,fin);
		*p = '\0';		/* terminate string */
		if ((s=lookup(sbuf)) == 0)
			s = install(sbuf,UNDEF,0.0);
		yylval.sym = s;
		return s->type == UNDEF ? VAR : s->type;
	}

	if (c == '$') {			/* argument */
		int n = 0;
		while (isdigit(c=getc(fin))) n = (n * 10) + (c - '0');
		ungetc(c,fin);
		if (n==0) execerror("strange $...", (char *) 0);
		yylval.narg = n;
		return ARG;
	}

	if (c == '"') {			/* quoted string */
		char sbuf[256], *p, *emalloc();
		for (p = sbuf; (c=getc(fin)) != '"'; p++) {
			if (c=='\n' || c == EOF) execerror("missing quote","");
			if (p>=sbuf+sizeof(sbuf) -1) {
				*p = '\0';
				execerror("string too long",sbuf);
			}
			*p = backslash(c); /* assign next char */
		}
		*p = 0;			/* terminate string */
		yylval.sym = (Symbol *)emalloc(strlen(sbuf)+1);
		strcpy(yylval.sym,sbuf);
		return STRING;
	}

	switch(c) {
	case '>':		return follow('=', GE, GT);
	case '<':		return follow('=', LE, LT);
	case '=':		return follow('=', EQ, '=');
	case '!':		return follow('=', NE, NOT);
	case '|':		return follow('|', OR, '|');
	case '&':		return follow('&', AND, '&');
	case '\n':		lineno++; return '\n';
	default:		return c;
	}
}

yyerror(s)				/* called for yacc syntax error */
char *s;
{
	warning(s,(char *) 0);
}

warning(s,t)	/* print warning message */
char *s, *t;
{
	fprintf(stderr,"%s: %s", progname, s);
	if (t) fprintf(stderr," %s", t);
	if (infile) fprintf(stderr, " in %s", infile);
	fprintf(stderr," near line %d\n", lineno);
	while (c != '\n' && c != EOF) {
		c = getc(fin);		/* flush rest of line */
	}
}

execerror(s,t)	/* recover from run time error */
char *s, *t;
{
	warning(s,t);			/* print error message */
	if (fin == stdin) fflush(stdin); /* flush if from terminal ... */
	else fseek(fin,0L,2);		/* ... or flush rest of file */
	longjmp(begin,0);		/* and return to 'safe' place */
}

fpecatch()	/* catch floating point exception */
{
	execerror("floating point exception", (char *) 0);
}

/*
 *				**** FOLLOW ****
 *
 * This routine looks ahead one character, then puts it back if
 *	it is not what was expected.
 * It returns the first argument if the character was found,
 *	otherwise it returns the second
 *
 */

follow(expect, ifyes, ifno)
int expect, ifyes, ifno;
{
	int c = getc(fin);

	if (c == expect) return ifyes;
	ungetc(c,fin);
	return ifno;
}

/*
 *				**** BACKSLASH ****
 *
 * This routine allows special characters to be quoted with back slashes
 *
 */

backslash(c)
int c;
{
	char	*strchr();
	static	char	transtab[] = "b\bf\fn\nr\rt\t";

	if (c != '\\') return c;	/* normal char, just return */
	c = getc(fin);			/* get 'escaped' character */
	/* if in translation table, return next character */
	if (islower(c) && strchr(transtab,c)) return strchr(transtab,c)[1];
	return c;			/* return character */
}

/*
 *				**** DEFNONLY ****
 *
 * This routine is called when encountering tokens that belong only within
 *	a definition.
 * It will produce a warning if not in one of them guys.
 *
 */

defnonly(s)
char *s;
{
	if (!indef) execerror(s,"used outside definition");
}
